// Cursor Demo C version
// Virtual-E (c) 1999-2000 <virtuale98@yahoo.com>
// Coded by Alberto Covarrubias
// (Modified by Amos Bieler [2002] for readability)

#include "vbaddr.h"	// You can use this header if you find it useful
#include "vbhw.h"	// This header is needed to use the default library
// #include "vbtypes.h"	// This header is included by vbhw.h because it must
			// be included if you want to use the default library

static HWORD chCursor[] = {
	0x0000,0x0500,0x5A50,0xAFA5,0xAFA5,0x5A50,0x0500,0x0000,
	0x0000,0x0050,0x05A5,0x5AFA,0x5AFA,0x05A5,0x0050,0x0000,
	0x03C0,0x0C30,0x300C,0x000C,0x000C,0x300C,0x0C30,0x03C0,
	0x300C,0x300C,0x300C,0x300C,0x300C,0x300C,0x0C30,0x03C0,
	0x03FC,0x0C0C,0x0C0C,0x0C0C,0x03FC,0x0C0C,0x300C,0x300C,
	0x0FF0,0x300C,0x000C,0x0FF0,0x3000,0x3000,0x300C,0x0FF0,
	0x03C0,0x0C30,0x300C,0x300C,0x300C,0x300C,0x0C30,0x03C0,
	0x03FC,0x0C0C,0x0C0C,0x0C0C,0x03FC,0x0C0C,0x300C,0x300C
};

static HWORD bgCursor[] = {0, 1, 2, 3, 4, 5, 6, 7};

// Set up the video hardware so the screen doesn't display the random, uninitialized VRAM.
void Init()
{
	VIP_REGS(BRTA)  = 0;	// Set the brightness registers to 0 (Black.)
	VIP_REGS(BRTB)  = 0;
	VIP_REGS(BRTC)  = 0;
	VIP_REGS(GPLT0) = 0xE4;	// Set all eight palettes to: 11, 10, 01, 00
	VIP_REGS(GPLT1) = 0xE4;	// (i.e. "Normal" dark to light progression.)
	VIP_REGS(GPLT2) = 0xE4;
	VIP_REGS(GPLT3) = 0xE4;
	VIP_REGS(JPLT0) = 0xE4;
	VIP_REGS(JPLT1) = 0xE4;
	VIP_REGS(JPLT2) = 0xE4;
	VIP_REGS(JPLT3) = 0xE4;
	VIP_REGS(BKCOL) = 0;	// Clear the screen to black before rendering
}

// This sets up and animates the bouncing cursor.  It never returns...
void Cursor()
{
	int x = 0;

	// Using another handy routine from the default library...
	// vbSetWorld (World, Header, GX, GP, GY, MX, MP, MY, Width, Height)
	// ...we can...

	// ...set up the first world (31) to hold the cursor...
	vbSetWorld(31,(WRLD_LON|WRLD_RON),0x60+x,2,0x6F,0,0,0,7,7);	
	// ...set up the second world (30) to display the word "CURSOR"...
	vbSetWorld(30,(WRLD_LON|WRLD_RON),0x78,2,0x60,16,0,0,47,7);	
	// ...and set up the third world (29) to be the End world.
	// This and all other worlds will not be drawn.
	vbSetWorld(29,WRLD_END,0,0,0,0,0,0,0,0);

	// See vbaddr.h for the constants (WRLD_*) from which to build a World's header.
	// Just OR them together like I did with LON and RON above.

	// Yet another "black box" function to handle the screen for us.
	vbDisplayOn();

	// Here's one that fades in the BRT* registers from black for a nice effect.
	vbFXFadeIn();

	// Everything inside this infinite while loop is the animation.
	// As you can see, the cursor moves to the right and, thanks
	// to lameboyadvance, flips horizontally to move back left!
	while(1)
	{
		for(x=0;x < 0x60;x++)
		{
			vbSetWorld(31,(WRLD_LON|WRLD_RON),0x60+x,2,0x6F,0,0,0,7,7);
			vbWaitFrame(15);
		}
		for(x=0x60;x > 0;x--)
		{
		// To perform the flip, the area being cut out of the BGMap is moved
		// 8 pixels (one tile) to the right, getting the reversed tile image, instead.
			vbSetWorld(31,(WRLD_LON|WRLD_RON),0x60+x,2,0x6F,8,0,0,7,7); 
			vbWaitFrame(15);
		// On a side note, instead of having a flipped version of the same tile
		// data twice, Alberto could have used the VB to flip it horizontally.
		// Then again, the code would have been harder to read...
		}
	}
}

// The compiler will accept only a void main()
// No return type or arguments allowed.
void main()
{
	int i=0;	// Just an example of declaring a variable.  i is never used...

	vbReset();	// Reset the VB (not sure exactly what this entails,
			// but it sets up the column tables, at least.)

	Init();		// Our screen initializing routine from above.

	// Copy the font chars into the char table and the string into the first BGMap.
	// This routine from the default library is very useful!

	// memcpy(destination address, source address, number of bytes)
	memcpy((void*)CharSeg0, (void*)chCursor, 8*16);	// CharSeg? returns the start address
							// of one of the CAM segments.
	memcpy((void*)BGMap(0), (void*)bgCursor, 16);	// BGMap(x) is a macro that returns
							// the address of one of the VB's
							// 14 BGMap segments.

	// Start the animation loop routine:
	Cursor();
}
